<?php

namespace Drupal\menu_link_content_view_access\Hook;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Session\AccountInterface;

/**
 * Contains hook implementations for the menu_link_content_view_access module.
 */
class MenuLinkContentViewAccessHooks {

  /**
   * Implements hook_ENTITY_TYPE_access().
   */
  #[Hook('menu_link_content_access')]
  public function menuLinkContentAccess(EntityInterface $entity, $operation, AccountInterface $account) {
    // Grant view access for our permission.
    if ($operation == 'view') {
      return AccessResult::allowedIf($account->hasPermission('access menu_link_content entities') && $entity->isPublished())->cachePerPermissions()->addCacheableDependency($entity);
    }
    else {
      // No opinion.
      return AccessResult::neutral();
    }
  }

  /**
   * Implements hook_jsonapi_ENTITY_TYPE_filter_access().
   */
  #[Hook('jsonapi_menu_link_content_filter_access')]
  public function jsonapiMenuLinkContentFilterAccess(EntityTypeInterface $entity_type, AccountInterface $account) {
    return ([
      JSONAPI_FILTER_AMONG_ALL => AccessResult::neutral(),
      JSONAPI_FILTER_AMONG_PUBLISHED => AccessResult::allowedIfHasPermission($account, 'access menu_link_content entities'),
      JSONAPI_FILTER_AMONG_OWN => AccessResult::allowedIfHasPermission($account, 'access menu_link_content entities'),
    ]);
  }

}
