<?php

namespace Drupal\Tests\menu_link_content_view_access\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\Tests\user\Traits\UserCreationTrait;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Test case class TODO.
 *
 * @group menu_link_content_view_access
 */
class MenuLinkContentViewAccessTest extends KernelTestBase {

  use UserCreationTrait;

  /**
   * The modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'system',
    'user',
    'link',
    'menu_link_content',
    'menu_ui',
    'file',
    'serialization',
    'jsonapi',
    'menu_link_content_view_access',
  ];

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The stacked http kernel service.
   *
   * @var \Symfony\Component\HttpKernel\HttpKernelInterface
   */
  protected $httpKernel;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->entityTypeManager = $this->container->get('entity_type.manager');
    $this->httpKernel = $this->container->get('http_kernel');

    $this->installSchema('user', ['users_data']);
    $this->installEntitySchema('user');

    $this->installEntitySchema('menu_link_content');
  }

  /**
   * Tests the menu link content view permission.
   */
  public function testMenuLinkContentViewAccess(): void {
    $menu_link = $this->entityTypeManager->getStorage('menu_link_content')->create([
      'link' => 'internal:#',
      'title' => 'Menu item',
      'enabled' => TRUE,
    ]);
    $menu_link->save();

    // A user with no access at all can't view a menu link over JSONAPI.
    $user_no_access = $this->setUpCurrentUser();

    $request = Request::create('/jsonapi/menu_link_content/menu_link_content/' . $menu_link->uuid());
    $response = $this->httpKernel->handle($request);
    $this->assertEquals(Response::HTTP_FORBIDDEN, $response->getStatusCode());

    // A user with our permission can view a menu link over JSONAPI, but not
    // access the admin UI.
    $user_with_permission = $this->setUpCurrentUser([], ['access menu_link_content entities']);

    $response = $this->httpKernel->handle($request);
    $this->assertEquals(Response::HTTP_OK, $response->getStatusCode());

    $request = Request::create('/admin/structure/menu');
    $response = $this->httpKernel->handle($request);
    $this->assertEquals(Response::HTTP_FORBIDDEN, $response->getStatusCode());

    $request = Request::create($menu_link->toUrl('edit-form')->getInternalPath());
    $response = $this->httpKernel->handle($request);
    $this->assertEquals(Response::HTTP_FORBIDDEN, $response->getStatusCode());

    // A user with the core admin permission can do everything.
    $user_with_admin = $this->setUpCurrentUser([], ['administer menu']);
    $response = $this->httpKernel->handle($request);
    $this->assertEquals(Response::HTTP_OK, $response->getStatusCode());

    $request = Request::create('/admin/structure/menu');
    $response = $this->httpKernel->handle($request);
    $this->assertEquals(Response::HTTP_OK, $response->getStatusCode());

    $request = Request::create($menu_link->toUrl('edit-form')->getInternalPath());
    $response = $this->httpKernel->handle($request);
    $this->assertEquals(Response::HTTP_OK, $response->getStatusCode());
  }

}
